syntax = "proto3";

package protobuf_unittest;

import "google/protobuf/descriptor.proto";

option java_outer_classname = "UnittestProto3Extensions";

// For testing proto3 extension behaviors.
message Proto3FileExtensions {
  extend google.protobuf.FileOptions {
    int32 singular_int = 1001;
    repeated int32 repeated_int = 1002;
  }
}
