// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// Author: kenton@google.com (Kenton Varda)
//
// Tests that a "lite" message can import a regular message.

syntax = "proto2";

package protobuf_unittest;

import "google/protobuf/unittest.proto";

option optimize_for = LITE_RUNTIME;

message TestLiteImportsNonlite {
  optional TestAllTypes message = 1;

  // Verifies that transitive required fields generates valid code.
  optional TestRequired message_with_required = 2;
}
