// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// These headers form a spanning tree for the upbc defs needed by FFI layers.

#include "upb_generator/get_used_fields.h"
#include "upb_generator/upbdev.h"
