// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// This file exists solely to document the google::protobuf::util namespace.
// It is not compiled into anything, but it may be read by an automated
// documentation generator.

namespace google {

namespace protobuf {

// Utility classes.
//
// This package contains various utilities for message comparison, JSON
// conversion, well known types, etc.
namespace util {}

}  // namespace protobuf
}  // namespace google
