// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto3";

package protobuf_unittest;

import "google/protobuf/any.proto";

option java_outer_classname = "TestAnyProto";

message TestAny {
  int32 int32_value = 1;
  google.protobuf.Any any_value = 2;
  repeated google.protobuf.Any repeated_any_value = 3;
  string text = 4;
}
